package metaedit;

import java.util.Observable;

import javax.swing.JMenuBar;

import locationapi.types.Area;
import metaconfig.types.ButtonArgs;
import metaconfig.types.MenuTitles;
import metaconfig.types.Properties;
import metaconfig.types.Property;
import metaedit.util.MenuBar;
import texteditorapi.types.Action;
import texteditorapi.types.Menu;
import aterm.ATerm;
import aterm.ATermList;

public class EditorModel extends Observable {
	private Menu menuEvent;

	private Action action;

	private Area area;

	private Property property;

	private String message;

	private String fileName;

	private locationapi.Factory locationFactory;

	private texteditorapi.Factory textFactory;

	private metaconfig.Factory metaconfigFactory;

	private aterm.pure.PureFactory pureFactory;

	private int beginFocus;

	private int focusLength;

	private MenuBar menuBar;

	private Properties properties;

	public EditorModel(aterm.pure.PureFactory pureFactory) {
		this.pureFactory = pureFactory;
		locationFactory = new locationapi.Factory(pureFactory);
		textFactory = new texteditorapi.Factory(pureFactory);
		metaconfigFactory = new metaconfig.Factory(pureFactory);
	}

	public void sendMessage(String message) {
		this.message = message;
		this.setChanged();
		this.notifyObservers("Message");
	}

	public void editFile(String fileName) {
		this.fileName = fileName;
		this.setChanged();
		this.notifyObservers("editFile");
	}

	public void rereadContents() {
		this.setChanged();
		this.notifyObservers("editFile");
	}

	public void writeContents() {
		this.setChanged();
		this.notifyObservers("writeContents");
	}

	public void setArea(ATerm area) {
		this.area = locationFactory.AreaFromTerm(area);
	}

	public void addActions(ATermList actionList, Controller controller) {
		menuBar = new MenuBar(pureFactory);
		menuBar.addMenu(actionList);
		this.setChanged();
		this.notifyObservers("addActions");
	}

	public void setFocus() {
		if (area.hasOffset() && area.hasLength()) {
			beginFocus = area.getOffset();
			focusLength = area.getLength();
			this.setChanged();
			this.notifyObservers("setFocus");
		}
	}

	public void setStyle() {
		this.setChanged();
		this.notifyObservers("setStyles");
	}

	public void clearFocus() {
		this.setChanged();
		this.notifyObservers("clearFocus");
	}

	public int getBeginFocus() {
		return beginFocus;
	}

	public int getFocusLength() {
		return focusLength;
	}

	public ATerm getMouseEvent(int event) {
		return textFactory.makeEvent_Mouse(event).toTerm();
	}

	public ATerm getMenuEvent(String action, String menuItem) {
		MenuTitles titles = metaconfigFactory.makeMenuTitles(action, menuItem);
		ButtonArgs menuEvent = metaconfigFactory.makeButtonArgs_Menu(titles);
		return menuEvent.toTerm();
	}

	public ATerm contentsChanged() {
		return textFactory.makeEvent_Modified().toTerm();
	}

	public JMenuBar getMenuBar() {
		return menuBar.getJMenuBar();
	}

	public String getMessage() {
		return message;
	}

	public String getFileName() {
		return fileName;
	}

}